package ru.dbotthepony.mc.otm.worldgen.placement

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.MapCodec
import net.minecraft.core.BlockPos
import net.minecraft.world.level.levelgen.placement.PlacementModifier
import ru.dbotthepony.mc.otm.worldgen.EnhancedPlacementContext

/**
 * Daisy-chaining placements. Required only when using placement modifiers which operate on children list
 */
class EnhancedChainPlacement(
	val children: List<EnhancedPlacementModifier>
) : EnhancedPlacementModifier {
	constructor(vararg children: EnhancedPlacementModifier) : this(ImmutableList.copyOf(children))

	override fun evaluate(context: EnhancedPlacementContext, positions: List<BlockPos>): List<BlockPos> {
		var current = positions
		children.forEach { current = it.evaluate(context, current) }
		return current
	}

	override val type: EnhancedPlacementModifier.Type<*>
		get() = Companion

	companion object : EnhancedPlacementModifier.Type<EnhancedChainPlacement> {
		override val codec: MapCodec<EnhancedChainPlacement> by lazy {
			Codec.list(EnhancedPlacementModifier.CODEC).xmap(::EnhancedChainPlacement, EnhancedChainPlacement::children).fieldOf("children")
		}
	}
}
